package java.util;

import java.security.PrivilegedAction;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import sun.util.calendar.ZoneInfo;
import java.time.ZoneId;
import sun.util.calendar.ZoneInfoFile;
import sun.util.locale.provider.TimeZoneNameUtility;
import java.io.Serializable;

public abstract class TimeZone implements Serializable, Cloneable
{
    public static final int SHORT = 0;
    public static final int LONG = 1;
    private static final int ONE_MINUTE = 60000;
    private static final int ONE_HOUR = 3600000;
    private static final int ONE_DAY = 86400000;
    static final long serialVersionUID = 3581463369166924961L;
    static final TimeZone NO_TIMEZONE;
    private String ID;
    private static volatile TimeZone defaultTimeZone;
    static final String GMT_ID = "GMT";
    private static final int GMT_ID_LENGTH = 3;
    private static volatile TimeZone mainAppContextDefault;
    
    public TimeZone() {
    }
    
    public abstract int getOffset(int p0, int p1, int p2, int p3, int p4, int p5);
    
    public int getOffset(long n) {
        if (this.inDaylightTime(new Date(n))) {
            return this.getRawOffset() + this.getDSTSavings();
        }
        return this.getRawOffset();
    }
    
    int getOffsets(long n, int[] array) {
        int rawOffset = this.getRawOffset();
        int dstSavings = 0;
        if (this.inDaylightTime(new Date(n))) {
            dstSavings = this.getDSTSavings();
        }
        if (array != null) {
            array[0] = rawOffset;
            array[1] = dstSavings;
        }
        return rawOffset + dstSavings;
    }
    
    public abstract void setRawOffset(int p0);
    
    public abstract int getRawOffset();
    
    public String getID() {
        return this.ID;
    }
    
    public void setID(String id) {
        if (id == null) {
            throw new NullPointerException();
        }
        this.ID = id;
        return;
    }
    
    public final String getDisplayName() {
        return this.getDisplayName(false, 1, Locale.getDefault(Locale.Category.DISPLAY));
    }
    
    public final String getDisplayName(Locale locale) {
        return this.getDisplayName(false, 1, locale);
    }
    
    public final String getDisplayName(boolean b, int n) {
        return this.getDisplayName(b, n, Locale.getDefault(Locale.Category.DISPLAY));
    }
    
    public String getDisplayName(boolean daylight, int n, Locale locale) {
        if (n != 0 && n != 1) {
            throw new IllegalArgumentException("Illegal style: " + n);
        }
        String id = this.getID();
        String retrieveDisplayName = TimeZoneNameUtility.retrieveDisplayName(id, daylight, n, locale);
        if (retrieveDisplayName != null) {
            return retrieveDisplayName;
        }
        if (id.startsWith("GMT") && id.length() > 3) {
            int char1 = id.charAt(3);
            if (char1 == '+' || char1 == '-') {
                return id;
            }
        }
        int rawOffset = this.getRawOffset();
        if (daylight) {
            rawOffset += this.getDSTSavings();
        }
        return ZoneInfoFile.toCustomID(rawOffset);
    }
    
    private static String[] getDisplayNames(String id, Locale locale) {
        return TimeZoneNameUtility.retrieveDisplayNames(id, locale);
    }
    
    public int getDSTSavings() {
        if (this.useDaylightTime()) {
            return 3600000;
        }
        return 0;
    }
    
    public abstract boolean useDaylightTime();
    
    public boolean observesDaylightTime() {
        return this.useDaylightTime() || this.inDaylightTime(new Date());
    }
    
    public abstract boolean inDaylightTime(Date p0);
    
    public static synchronized TimeZone getTimeZone(String s) {
        return getTimeZone(s, true);
    }
    
    public static TimeZone getTimeZone(ZoneId zoneId) {
        String str = zoneId.getId();
        int char1 = str.charAt(0);
        if (char1 == '+' || char1 == '-') {
            str = "GMT" + str;
        }
        else if (char1 == 'Z' && str.length() == 1) {
            str = "UTC";
        }
        return getTimeZone(str, true);
    }
    
    public ZoneId toZoneId() {
        String id = this.getID();
        if (ZoneInfoFile.useOldMapping() && id.length() == 3) {
            if ("EST".equals(id)) {
                return ZoneId.of("America/New_York");
            }
            if ("MST".equals(id)) {
                return ZoneId.of("America/Denver");
            }
            if ("HST".equals(id)) {
                return ZoneId.of("America/Honolulu");
            }
        }
        return ZoneId.of(id, ZoneId.SHORT_IDS);
    }
    
    private static TimeZone getTimeZone(String id, boolean b) {
        TimeZone timeZone = ZoneInfo.getTimeZone(id);
        if (timeZone == null) {
            timeZone = parseCustomTimeZone(id);
            if (timeZone == null && b) {
                timeZone = new ZoneInfo("GMT", 0);
            }
        }
        return timeZone;
    }
    
    public static synchronized String[] getAvailableIDs(int rawOffset) {
        return ZoneInfo.getAvailableIDs(rawOffset);
    }
    
    public static synchronized String[] getAvailableIDs() {
        return ZoneInfo.getAvailableIDs();
    }
    
    private static native String getSystemTimeZoneID(String p0);
    
    private static native String getSystemGMTOffsetID();
    
    public static TimeZone getDefault() {
        return (TimeZone)getDefaultRef().clone();
    }
    
    static TimeZone getDefaultRef() {
        TimeZone timeZone = TimeZone.defaultTimeZone;
        if (timeZone == null) {
            timeZone = setDefaultZone();
            assert timeZone != null;
        }
        return timeZone;
    }
    
    private static synchronized TimeZone setDefaultZone() {
        final String systemTimeZoneID = (String)AccessController.doPrivileged(new GetPropertyAction("user.timezone"));
        if (systemTimeZoneID == null || systemTimeZoneID.isEmpty()) {
            String s = (String)AccessController.doPrivileged(new GetPropertyAction("java.home"));
            try {
                systemTimeZoneID = getSystemTimeZoneID(s);
                if (systemTimeZoneID == null) {
                    systemTimeZoneID = "GMT";
                }
            }
            catch (NullPointerException ex) {
                systemTimeZoneID = "GMT";
            }
        }
        TimeZone defaultTimeZone = getTimeZone(systemTimeZoneID, false);
        if (defaultTimeZone == null) {
            String systemGMTOffsetID = getSystemGMTOffsetID();
            if (systemGMTOffsetID != null) {
                systemTimeZoneID = systemGMTOffsetID;
            }
            defaultTimeZone = getTimeZone(systemTimeZoneID, true);
        }
        assert defaultTimeZone != null;
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                System.setProperty("user.timezone", systemTimeZoneID);
                return null;
            }
        });
        return TimeZone.defaultTimeZone = defaultTimeZone;
    }
    
    public static void setDefault(TimeZone defaultTimeZone) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new PropertyPermission("user.timezone", "write"));
        }
        TimeZone.defaultTimeZone = defaultTimeZone;
    }
    
    public boolean hasSameRules(TimeZone timeZone) {
        return timeZone != null && this.getRawOffset() == timeZone.getRawOffset() && this.useDaylightTime() == timeZone.useDaylightTime();
    }
    
    public Object clone() {
        try {
            TimeZone timeZone = (TimeZone)super.clone();
            timeZone.ID = this.ID;
            return timeZone;
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause);
        }
    }
    
    private static final TimeZone parseCustomTimeZone(String s) {
        int length;
        if ((length = s.length()) < 5 || s.indexOf("GMT") != 0) {
            return null;
        }
        ZoneInfo zoneInfo = ZoneInfoFile.getZoneInfo(s);
        if (zoneInfo != null) {
            return zoneInfo;
        }
        int i = 3;
        boolean n = false;
        int char1 = s.charAt(i++);
        if (char1 == '-') {
            n = true;
        }
        else if (char1 != '+') {
            return null;
        }
        int n2 = 0;
        int n3 = 0;
        int n4 = 0;
        int n5 = 0;
        while (i < length) {
            int char2 = s.charAt(i++);
            if (char2 == ':') {
                if (n4 > 0) {
                    return null;
                }
                if (n5 > 2) {
                    return null;
                }
                n2 = n3;
                ++n4;
                n3 = 0;
                n5 = 0;
            }
            else {
                if (char2 < '0' || char2 > '9') {
                    return null;
                }
                n3 = n3 * 10 + (char2 - 48);
                ++n5;
            }
        }
        if (i != length) {
            return null;
        }
        if (n4 == 0) {
            if (n5 <= 2) {
                n2 = n3;
                n3 = 0;
            }
            else {
                n2 = n3 / 100;
                n3 %= 100;
            }
        }
        else if (n5 != 2) {
            return null;
        }
        if (n2 > 23 || n3 > 59) {
            return null;
        }
        int n6 = (n2 * 60 + n3) * 60 * 1000;
        ZoneInfo zoneInfo2;
        if (n6 == 0) {
            zoneInfo2 = ZoneInfoFile.getZoneInfo("GMT");
            if (!(n == 0)) {
                zoneInfo2.setID("GMT-00:00");
            }
            else {
                zoneInfo2.setID("GMT+00:00");
            }
        }
        else {
            zoneInfo2 = ZoneInfoFile.getCustomTimeZone(s, n ? (-n6) : n6);
        }
        return zoneInfo2;
    }
    
    static {
        NO_TIMEZONE = null;
    }
}
